# Copyright Louis Paternault 2011-2026
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>. 1

"""Calendar generation"""

import typing

import jinja2

from .. import __DATE__, VERSION

if typing.TYPE_CHECKING:
    from ..calendar import Calendar


def generate_tex(calendar: "Calendar") -> str:
    """Generate TeX code producing calendar represented in argument.

    :arg Calendar calendar: A :class:`Calendar` object.
    """
    loader = jinja2.ChoiceLoader(
        [
            jinja2.PackageLoader("scal.data"),
            jinja2.FileSystemLoader("."),
        ]
    )

    environment = jinja2.Environment(loader=loader)
    environment.block_start_string = "(*"
    environment.block_end_string = "*)"
    environment.variable_start_string = "(("
    environment.variable_end_string = "))"
    environment.comment_start_string = "(% comment %)"
    environment.comment_end_string = "(% endcomment %)"
    environment.line_comment_prefix = "%!"
    # environment.filters['escape_tex'] = _escape_tex
    environment.trim_blocks = True
    environment.lstrip_blocks = True
    return environment.get_template(calendar.template).render(
        {  # pylint: disable=maybe-no-member
            "start": calendar.start,
            "end": calendar.end,
            "count": {
                "months": calendar.months_count(),
                "weeks": calendar.weeks_count(),
            },
            "holidays": calendar.holidays,
            "weeks": calendar.weeks,
            "years": calendar.year_boundaries(),
            "version": f"`scal` version {VERSION}",
            "variables": calendar.variables,
            "copyrightdate": __DATE__,
        }
    )



#!/usr/bin/env python3

# Copyright Louis Paternault 2013-2026
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>. 1

"""Generate the configuration file to build a calendar with Wikimedia images"""

# pylint: disable=invalid-name

import argparse
import contextlib
import datetime
import pathlib
import random
import subprocess
import sys
import tempfile
import textwrap
import time

import jinja2
import requests
from rich.console import Console
from rich.progress import track

PROGRAM_NAME = __file__
MAX_RETRYS = 10
WIKIMEDIA_API_URL = (
    "https://api.wikimedia.org/feed/v1/wikipedia/{language}/featured/{date}"
)

USAGE = textwrap.dedent(f"""\
        {__doc__}

        Usage:

        {PROGRAM_NAME} LANGUAGE YEAR

        The output is a configuration file to be processed by `scal`…
        """)

TEMPLATE = jinja2.Template(textwrap.dedent("""\
    calendar:
      template: monthly.tex
      start: {{ year }}-01-01
      end: {{ year }}-12-31

    variables:
      language: {{ language }}
      pictures:
        {{ pictures }}
    """))

CONSOLE = Console(stderr=True)

# The wikimedia API has a limit of 500 requests per hour, or one request every 7.2 seconds
# Let's use 8 seconds to be safe.
# https://api.wikimedia.org/wiki/Rate_limits
MINIMUM_DELAY = 8


class WikimediaDownloader:
    """Download data from Wikimedia."""

    def __init__(self, dest):
        self.dest = dest
        dest.mkdir(exist_ok=True)
        self._last_call = datetime.datetime.now() - datetime.timedelta(
            seconds=MINIMUM_DELAY
        )

    @contextlib.contextmanager
    def sleep(self):
        """Wait `MINIMUM_DELAY` between two calls."""
        sleep = max(
            0,
            (
                self._last_call
                + datetime.timedelta(seconds=MINIMUM_DELAY)
                - datetime.datetime.now()
            ).total_seconds(),
        )
        time.sleep(sleep)
        yield from [None]
        self._last_call = datetime.datetime.now()

    @property
    def missing(self):
        """Return number of missing images.

        We want 12 pictures in the destination directory (one per month).
        """
        return 12 - len(list(self.dest.glob("*.jpg")))

    def picturename(self, date, suffix):
        """Retun the name of the file in which the picture of the given date is stored."""
        return (self.dest / date.strftime("%Y%m%d")).with_suffix(suffix)

    def download_picture(self):
        """Download the picture of the day of a random date.

        Save the image in file given by :func:`picturename`.
        Return the (date, credit) of the picture.
        """
        errors = 0

        while True:
            # Repeat until image have been successfully downloaded
            date = random_date()
            if self.picturename(date, ".jpg").exists():
                continue

            try:
                # Download image metadata
                CONSOLE.log(f"""Downloading image metadata for date {date}.""")
                response = self.download(
                    WIKIMEDIA_API_URL.format(
                        language="en", date=date.strftime("%Y/%m/%d")
                    )
                ).json()

                # pylint: disable=line-too-long, consider-using-f-string
                credit = """*{title}*, by {artist}, published under licence {licence}\n\n[{url}]({url})""".format(
                    title=response["image"]["description"]["text"],
                    artist=response["image"]["artist"]["text"],
                    licence=response["image"]["license"]["type"],
                    url=response["image"]["file_page"],
                )

                if (
                    response["image"]["image"]["width"]
                    > 2 * response["image"]["image"]["height"]
                ):
                    # Images much wider than tall won't render well in the calendar
                    CONSOLE.log(
                        # pylint: disable=line-too-long
                        f"Image too wide ({response["image"]["image"]["width"]}x{response["image"]["image"]["height"]}). Trying another image…"
                    )
                    continue

                if (
                    response["image"]["image"]["width"] > 32000
                    or response["image"]["image"]["height"] > 32000
                ):
                    # Mogrify (called later) cannot process images too big.
                    CONSOLE.log(
                        # pylint: disable=line-too-long
                        f"Image too big ({response["image"]["image"]["width"]}x{response["image"]["image"]["height"]}). Trying another image…"
                    )
                    continue

                # Download actual image
                CONSOLE.log(f"""Downloading image for date {date}.""")
                with tempfile.NamedTemporaryFile(
                    delete_on_close=False,
                    suffix=pathlib.Path(response["image"]["image"]["source"]).suffix,
                ) as file:
                    # Download file
                    for chunk in self.download(
                        response["image"]["image"]["source"], stream=True
                    ).iter_content(chunk_size=128):
                        file.write(chunk)
                    file.close()

                    # Rename it to a proper image, while:
                    # - converting it to jpg (with a ".jpg" suffix),
                    # - reducing it if too big,
                    # - setting density to 300dpi (this should be useless, because the image is resized anyway, but some image have a faulty density of 1dpi, which give huge dimensions, which exceeds LaTeX maximum dimension).
                    subprocess.run(
                        [
                            "convert",
                            file.name,
                            "-resize",
                            "2000x2000>",
                            "-density",
                            "300",
                            self.picturename(date, ".jpg"),
                        ],
                        stderr=subprocess.PIPE,
                        check=True,
                    )
                # Write credit into an associated markdown file
                with open(
                    self.picturename(date, ".mdwn"), mode="w", encoding="utf8"
                ) as file:
                    file.write(credit)
            except (KeyError, requests.exceptions.HTTPError) as error:
                CONSOLE.log(
                    f"Error while downloading picture of the day {date}: {error}."
                )
                if errors >= MAX_RETRYS:
                    CONSOLE.log("Too many errors. Aborting.")
                    sys.exit(1)
                continue

            # Image downloaded sucessfully
            break

        # Build attribution text (in LaTeX)
        # pylint: disable=line-too-long
        return (date, credit)

    def download(self, url, stream=False):
        """Download the content at the given URL."""
        with self.sleep():
            response = requests.get(
                url,
                stream=stream,
                headers={"User-Agent": PROGRAM_NAME},
                timeout=60,
            )
            try:
                response.raise_for_status()
            except requests.exceptions.HTTPError:
                if response.status_code == 429:
                    CONSOLE.log(
                        # pylint: disable=line-too-long
                        f"""Error 429: Server is asking us to wait {response.headers["retry-after"]} seconds…"""
                    )
                    # The server kindly requests us to stop downloading for a while.
                    time.sleep(float(response.headers["retry-after"]))
                raise
            return response


def random_date():
    """Returns a random date since Wikipedia started having "pictures of the day"."""
    # I do not know exactly what is the oldest available data
    lower = datetime.date(2016, 1, 1)
    upper = datetime.date.today()
    return lower + datetime.timedelta(days=random.randint(0, (upper - lower).days))


def escape(string):
    """Escape the HTML string so that it can be rendered by LaTeX."""
    return string.replace("%", r"\%").replace("&amp;", r"\&").replace("&nbsp;", " ")


def main():
    """Main function"""

    # Downloading pictures
    downloader = WikimediaDownloader(argument_parser().parse_args().DEST)
    for _ in track(
        range(downloader.missing),
        description="Downloading images…",
        console=CONSOLE,
        transient=True,
    ):
        downloader.download_picture()


def argument_parser():
    """Return a command line parser."""
    parser = argparse.ArgumentParser(
        prog=__file__,
        description="Download random pictures of the day from Wikimedia",
    )

    parser.add_argument(
        "DEST",
        type=pathlib.Path,
        default=".",
        help="Destination directory.",
    )

    return parser


if __name__ == "__main__":
    main()



#!/usr/bin python

# Copyright 2015-2026 Louis Paternault
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Tests"""

import doctest
import pkgutil
import sys

import scal


def load_tests(__loader, tests, __pattern):  # type: ignore
    """Load tests (doctests)."""
    # Loading doctests
    tests.addTests(doctest.DocTestSuite(scal))
    for module_finder, name, __is_pkg in pkgutil.walk_packages(
        scal.__path__, prefix=f"{scal.__name__}."
    ):
        if name in sys.modules:
            module = sys.modules[name]
        else:
            try:
                module = module_finder.find_spec(name).loader.load_module()  # type: ignore
            except ImportError:
                continue
        try:
            tests.addTests(doctest.DocTestSuite(module))
        except ValueError:
            # No docstring, or no doctests in the docstrings
            pass

    return tests



# Copyright 2019-2023 Louis Paternault
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero Public License for more details.
#
# You should have received a copy of the GNU Affero Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Automatically generate and overwrite examples of .scl files.

`autoautoscl --help` for more information.
"""

import argparse
import datetime
import logging
import os
import re
import shlex
import subprocess
import sys
import textwrap
import time

logging.basicConfig(level=logging.INFO)

RE_YEARRANGE = re.compile(r"(\d{4})-(\d{4})", re.ASCII)

REPOROOT = os.path.abspath(os.path.join(os.path.dirname(__file__), "..", "..", ".."))

CURRENTYEAR = int(datetime.date.today().strftime("%Y"))


class Error(Exception):
    """Exceptions to be catched and nicely formatted to the user."""


def auto_fr_educnat(years):
    """Iterate over commands and destination files to run (French Éducation nationale)

    Yields command to run, as a list to be passed to subprocess.run()
      (working directory being the repository root).
    """
    for year in years:
        for zone, ville in (("A", "Grenoble"), ("B", "Rennes"), ("C", "Paris")):
            yield [
                "./bin/autoscl",
                "--templates",
                "calendar",
                "weekly",
                "--output",
                f"doc/examples/{{template}}-fr-{year}{year+1}-{zone}.scl",
                "fr.educnat",
                f"{year}-{year+1}",
                ville,
            ]


def auto_en(years):
    """Iterate over commands and destination files to run (English)"""
    for year in years:
        yield [
            "./bin/autoscl",
            "--templates",
            "calendar",
            "weekly",
            "--output",
            f"doc/examples/{{template}}-en-{year}{year+1}.scl",
            "en",
            f"{year}-{year+1}",
        ]


COUNTRIES = {
    "fr.educnat": auto_fr_educnat,
    "en": auto_en,
}


def _type_country(text):
    """Check that country is supported."""
    if text in COUNTRIES:
        return text
    raise argparse.ArgumentTypeError(
        # pylint: disable=consider-using-f-string
        "{} is not a valid countries. Choose a string among: {}.".format(
            text, ", ".join(COUNTRIES.keys())
        )
    )


def _type_yearrange(text):
    """Check that year is a range of years (e.g. "2015-2020")."""
    match = RE_YEARRANGE.match(text)
    if not match:
        raise argparse.ArgumentTypeError(
            f"{text} is not a valid year range: it must be of the form YYYY-YYYY (e.g. 2015-2020)."
        )
    start, end = int(match.groups()[0]), int(match.groups()[1])
    return list(range(start, end))


def argumentparser():
    """Return an argument parser."""
    # pylint: disable=line-too-long
    parser = argparse.ArgumentParser(
        description="Generate and overwrite .scl examples.",
        formatter_class=argparse.RawDescriptionHelpFormatter,
        epilog=textwrap.dedent("""\
                Example: 'autoautoscl -c fr.educnat 2017-2020' generates the .scl files for French official school years, from 2017 to 2020.

                <french>
                Pour générer les calendriers des trois zones, depuis la racine du projet, utiliser (ajouter éventuellement les années):

                ./bin/autoautoscl
                ./bin/generate_examples.sh

                </french>
                """),
    )
    parser.add_argument(
        "-c",
        "--countries",
        help="Countries of calendar.",
        type=_type_country,
        nargs="+",
        default=list(COUNTRIES.keys()),
    )
    parser.add_argument(
        "years",
        help="Calendar school years.",
        nargs="?",
        type=_type_yearrange,
        default=f"{CURRENTYEAR}-{CURRENTYEAR+1}",
    )
    return parser


def main():
    """Main function."""
    args = argumentparser().parse_args()
    for country in args.countries:
        print("Country:", country)
        for command in COUNTRIES[country](args.years):
            print(shlex.join(command), end=" # ")
            if subprocess.call(command, cwd=REPOROOT) == 0:
                print("SUCCESS")
            else:
                print("ERROR")
            time.sleep(2)  # Be kind to the opendata servers.


if __name__ == "__main__":
    try:
        if sys.version_info < (3, 8):
            raise Error("This program requires python version 3.8 or above.")
        main()
    except Error as error:
        logging.error(str(error))
        sys.exit(1)



#!/usr/bin/env python3

# Copyright Louis Paternault 2011-2026
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>. 1

"""Produce a monthly calendar with pictures."""

import argparse
import datetime
import pathlib
import shutil
import subprocess
import tempfile

import papersize
from PIL import Image

from .. import calendar, template


def argument_parser() -> argparse.ArgumentParser:
    """Return a command line parser."""
    # pylint: disable=line-too-long

    parser = argparse.ArgumentParser(
        prog="scal",
        description="A year calendar producer.",
    )

    parser.add_argument(
        "-y",
        "--year",
        type=int,
        default=datetime.datetime.today().year,
        help=f"Year of the calendar (e.g. {datetime.datetime.today().year})",
    )
    parser.add_argument(
        "-l",
        "--language",
        type=str,
        default="english",
        help="""Calendar language (e.g. "french", "english", "spanish"…).""",
    )
    parser.add_argument(
        "-p",
        "--papersize",
        default="A4",
        type=papersize.parse_papersize,
        help="""Size of the paper (e.g. "A4", "21cm×297mm", etc.).""",
    )
    parser.add_argument(
        "-o",
        "--output",
        default=None,
        type=pathlib.Path,
        help="""Output file. Default is the year, given by option `--year` (e.g. "{datetime.datetime.today().year}.pdf").""",
    )
    parser.add_argument(
        "PICTURE",
        type=pathlib.Path,
        nargs=12,
        help="File names of twelve pictures (one for each month).",
    )

    return parser


def _parse_picture(filename: pathlib.Path) -> dict[str, str | pathlib.Path]:
    # Look for credits
    if filename.with_suffix(".mdwn").exists():
        credit = subprocess.run(
            [
                "pandoc",
                "--from",
                "markdown",
                "--to",
                "latex",
                filename.with_suffix(".mdwn"),
            ],
            stdin=subprocess.DEVNULL,
            check=True,
            stdout=subprocess.PIPE,
            text=True,
        ).stdout
    else:
        credit = ""

    image = Image.open(filename)
    if image.size[0] > image.size[1]:
        orientation = "horizontal"
    else:
        orientation = "vertical"
    # Check image size
    return {
        "filename": filename.resolve(),
        "orientation": orientation,
        "credit": credit,
    }


def main() -> None:
    """Main function."""
    # Get command line options
    options = argument_parser().parse_args()
    if options.output is None:
        options.output = pathlib.Path(f"{options.year}.pdf")

    with tempfile.TemporaryDirectory() as tempdir:
        # Build LaTeX file
        with open(
            pathlib.Path(tempdir) / f"{options.year}.tex", mode="w", encoding="utf8"
        ) as file:
            file.write(
                template.generate_tex(
                    calendar.Calendar(
                        {
                            "calendar": {
                                "template": "monthly.tex",
                                "start": datetime.date(
                                    year=options.year, month=1, day=1
                                ),
                                "end": datetime.date(
                                    year=options.year, month=12, day=1
                                ),
                            },
                            "variables": {
                                # pylint: disable=line-too-long
                                "language": options.language,
                                "papersize": f"paperwidth={round(options.papersize[0])}pt, paperheight={round(options.papersize[1])}pt",
                                "pictures": [
                                    _parse_picture(filename)
                                    for filename in options.PICTURE
                                ],
                            },
                        }
                    )
                )
            )
        # Compile file
        for _ in range(3):
            subprocess.run(
                ["lualatex", str(options.year)],
                cwd=tempdir,
                stdin=subprocess.DEVNULL,
                check=True,
            )

        # Get resulting file
        shutil.move(pathlib.Path(tempdir) / f"{options.year}.pdf", options.output)


if __name__ == "__main__":
    main()
